@@ -422,14 +422,14 @@ struct tTLV {
 	unsigned char *next;
 };
 
-static inline void TLVInit(struct tTLV *tlv, u8 * base, size_t size)
+static void TLVInit(struct tTLV *tlv, u8 * base, size_t size)
 {
 	tlv->base = base;
 	tlv->end = base + size;
 	tlv->current = tlv->next = base;
 }
 
-static inline void TLVNext(struct tTLV *tlv, u8 tag)
+static void TLVNext(struct tTLV *tlv, u8 tag)
 {
 	assert(tlv->next + 2 < tlv->end);
 	tlv->current = tlv->next;
@@ -437,14 +437,14 @@ static inline void TLVNext(struct tTLV *tlv, u8 tag)
 	*(tlv->next++) = 0;
 }
 
-static inline void TLVAdd(struct tTLV *tlv, u8 val)
+static void TLVAdd(struct tTLV *tlv, u8 val)
 {
 	assert(tlv->next + 1 < tlv->end);
 	*(tlv->next++) = val;
 	tlv->current[1]++;
 }
 
-static inline void TLVAddBuffer(struct tTLV *tlv, u8 * val, size_t size)
+static void TLVAddBuffer(struct tTLV *tlv, u8 * val, size_t size)
 {
 	assert(tlv->next + size < tlv->end);
 	memcpy(tlv->next, val, size);
@@ -647,7 +647,7 @@ int belpic_set_language(const char *reader, int lang)
 				lang_infos[i].reader[0] = '\0';
 		} else {	/* For only 1 reader */
 			if (lang_infos[i].reader[0] == '\0') {	/* reader not yet present */
-				strncpy(lang_infos[i].reader, reader, MAX_READER_LEN - 1);
+				strlcpy(lang_infos[i].reader, reader, sizeof(lang_infos[i].reader));
 				lang_infos[i].lang = lang;
 				break;
 			} else if (strncmp(reader, lang_infos[i].reader, MAX_READER_LEN - 1) == 0) {
@@ -1536,9 +1536,9 @@ static int belpic_compute_signature(sc_card_t *card, const u8 * data,
 	return r;
 }
 
+#if 0
 static int belpic_logout(sc_card_t *card)
 {
-#if 0
 	sc_apdu_t apdu;
 	int r;
 
@@ -1552,9 +1552,8 @@ static int belpic_logout(sc_card_t *card)
 	SC_TEST_RET(card->ctx, r, "LOGOFF returned error");
 
 	SC_FUNC_RETURN(card->ctx, 1, r);
-#endif
-	return 0;
 }
+#endif
 
 static struct sc_card_driver *sc_get_driver(void)
 {
@@ -1569,7 +1568,6 @@ static struct sc_card_driver *sc_get_driver(void)
 	belpic_ops.read_binary = belpic_read_binary;
 	belpic_ops.pin_cmd = belpic_pin_cmd;
 	belpic_ops.set_security_env = belpic_set_security_env;
-	belpic_ops.logout = belpic_logout;
 
 	belpic_ops.compute_signature = belpic_compute_signature;
 	belpic_ops.get_challenge = iso_ops->get_challenge;
