@@ -147,6 +147,7 @@ extern "C" {
 #define SC_ALGORITHM_RSA		0
 #define SC_ALGORITHM_DSA		1
 #define SC_ALGORITHM_EC			2
+#define SC_ALGORITHM_GOSTR3410		3
 
 /* Symmetric algorithms */
 #define SC_ALGORITHM_DES		64
@@ -156,7 +157,7 @@ extern "C" {
 /* Hash algorithms */
 #define SC_ALGORITHM_MD5		128
 #define SC_ALGORITHM_SHA1		129
-#define SC_ALGORITHM_GOSTHASH		130
+#define SC_ALGORITHM_GOSTR3411		130
 
 /* Key derivation algorithms */
 #define SC_ALGORITHM_PBKDF2		192
@@ -190,9 +191,10 @@ extern "C" {
 #define SC_ALGORITHM_RSA_HASH_SHA224	0x00001000
 #define SC_ALGORITHM_RSA_HASHES		0x00001FE0
 
-#define SC_ALGORITHM_GOST_CRYPT_PZ     0x0     
-#define SC_ALGORITHM_GOST_CRYPT_GAMM   0x1     
-#define SC_ALGORITHM_GOST_CRYPT_GAMMOS 0x2     
+#define SC_ALGORITHM_GOSTR3410_RAW		0x00002000
+#define SC_ALGORITHM_GOSTR3410_HASH_NONE	0x00004000
+#define SC_ALGORITHM_GOSTR3410_HASH_GOSTR3411	0x00008000
+#define SC_ALGORITHM_GOSTR3410_HASHES		0x00008000
 
 /* Event masks for sc_wait_for_event() */
 #define SC_EVENT_CARD_INSERTED		0x0001
@@ -681,6 +683,7 @@ int sc_transmit_apdu(sc_card_t *card, sc_apdu_t *apdu);
 void sc_format_apdu(sc_card_t *card, sc_apdu_t *apdu, int cse, int ins,
 		    int p1, int p2);
 
+int sc_check_sw(struct sc_card *card, unsigned int sw1, unsigned int sw2);
 
 /********************************************************************/
 /*                  opensc context functions                        */
