@@ -194,6 +194,7 @@ static CK_RV pkcs15_unbind(struct sc_pkcs11_card *p11card)
 	unlock_card(fw_data);
 
 	rc = sc_pkcs15_unbind(fw_data->p15_card);
+	free(fw_data);
 	return sc_to_cryptoki_error(rc, p11card->reader);
 }
 
@@ -272,6 +273,24 @@ __pkcs15_release_object(struct pkcs15_any_object *obj)
 	return 0;
 }
 
+static int
+__pkcs15_delete_object(struct pkcs15_fw_data *fw_data, struct pkcs15_any_object *obj)
+{
+	unsigned int i;
+
+	if (fw_data->num_objects == 0)
+		return SC_ERROR_INTERNAL;
+
+	for (i = 0; i < fw_data->num_objects; ++i)
+		if (fw_data->objects[i] == obj) {
+			fw_data->objects[i] = fw_data->objects[--fw_data->num_objects];
+			if (__pkcs15_release_object(obj) > 0)
+				return SC_ERROR_INTERNAL;
+			return SC_SUCCESS;
+		}
+	return SC_ERROR_OBJECT_NOT_FOUND;
+}
+
 static int public_key_created(struct pkcs15_fw_data *fw_data,
 			      const unsigned int num_objects,
 			      const u8 *id, 
@@ -923,6 +942,7 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 static CK_RV pkcs15_release_token(struct sc_pkcs11_card *p11card, void *fw_token)
 {
 	unlock_card((struct pkcs15_fw_data *) p11card->fw_data);
+	free(fw_token);
 	return CKR_OK;
 }
 
@@ -1739,6 +1759,10 @@ static CK_RV pkcs15_gen_keypair(struct sc_pkcs11_card *p11card,
 		if (rc >= 0)
 			rc = sc_pkcs15init_store_public_key(p15card, profile,
 				&pub_args, &pub_key_obj);
+
+		sc_pkcs15_erase_prkey(&keygen_args.prkey_args.key);
+		sc_pkcs15_erase_pubkey(&pub_args.key);
+
 		if (rc < 0) {
 			sc_debug(context, "private/public keys not stored: %d\n", rc);
 			rv = sc_to_cryptoki_error(rc, p11card->reader);
@@ -2169,6 +2193,12 @@ static CK_RV pkcs15_prkey_get_attribute(struct sc_pkcs11_session *session,
 	case CKA_END_DATE:
 		attr->ulValueLen = 0;
 		return CKR_OK;
+	case CKA_GOSTR3410_PARAMS:
+		if (prkey->prv_info && prkey->prv_info->params_len)
+			return get_gostr3410_params(prkey->prv_info->params,
+					prkey->prv_info->params_len, attr);
+		else
+			return CKR_ATTRIBUTE_TYPE_INVALID;
 	default:
 		return CKR_ATTRIBUTE_TYPE_INVALID;
 	}
@@ -2724,6 +2754,13 @@ static CK_RV pkcs15_dobj_destroy(struct sc_pkcs11_session *session, void *object
 		if (rv == 0)
 			rv = sc_pkcs15init_delete_object(fw_data->p15_card, profile, obj->base.p15_object);
 	}
+	if (rv >= 0) {
+		/* pool_find_and_delete is called, therefore correct refcont
+		 * Oppose to pkcs15_add_object */
+		--((struct pkcs15_any_object*)object)->refcount;
+		/* Delete object in pkcs15 */
+		rv = __pkcs15_delete_object(fw_data, (struct pkcs15_any_object*)object);
+	}
 
 	sc_pkcs15init_unbind(profile);
 	sc_unlock(card->card);
