@@ -289,12 +289,27 @@ static enum_specs ck_mec_s[] = {
   { CKM_SHA_1                    , "CKM_SHA_1                    " },
   { CKM_SHA_1_HMAC               , "CKM_SHA_1_HMAC               " },
   { CKM_SHA_1_HMAC_GENERAL       , "CKM_SHA_1_HMAC_GENERAL       " },
+  { CKM_SHA256                   , "CKM_SHA256                   " },
+  { CKM_SHA256_HMAC              , "CKM_SHA256_HMAC              " },
+  { CKM_SHA256_HMAC_GENERAL      , "CKM_SHA256_HMAC_GENERAL      " },
+  { CKM_SHA384                   , "CKM_SHA384                   " },
+  { CKM_SHA384_HMAC              , "CKM_SHA384_HMAC              " },
+  { CKM_SHA384_HMAC_GENERAL      , "CKM_SHA384_HMAC_GENERAL      " },
+  { CKM_SHA512                   , "CKM_SHA512                   " },
+  { CKM_SHA512_HMAC              , "CKM_SHA512_HMAC              " },
+  { CKM_SHA512_HMAC_GENERAL      , "CKM_SHA512_HMAC_GENERAL      " },
   { CKM_RIPEMD128                , "CKM_RIPEMD128                " },
   { CKM_RIPEMD128_HMAC           , "CKM_RIPEMD128_HMAC           " },
   { CKM_RIPEMD128_HMAC_GENERAL   , "CKM_RIPEMD128_HMAC_GENERAL   " },
   { CKM_RIPEMD160                , "CKM_RIPEMD160                " },
   { CKM_RIPEMD160_HMAC           , "CKM_RIPEMD160_HMAC           " },
   { CKM_RIPEMD160_HMAC_GENERAL   , "CKM_RIPEMD160_HMAC_GENERAL   " },
+  { CKM_SHA256                   , "CKM_SHA256                   " },
+  { CKM_SHA256_HMAC              , "CKM_SHA256_HMAC              " },
+  { CKM_SHA256_HMAC_GENERAL      , "CKM_SHA256_HMAC_GENERAL      " },
+  { CKM_SHA384                   , "CKM_SHA384                   " },
+  { CKM_SHA384_HMAC              , "CKM_SHA384_HMAC              " },
+  { CKM_SHA384_HMAC_GENERAL      , "CKM_SHA384_HMAC_GENERAL      " },
   { CKM_CAST_KEY_GEN             , "CKM_CAST_KEY_GEN             " },
   { CKM_CAST_ECB                 , "CKM_CAST_ECB                 " },
   { CKM_CAST_CBC                 , "CKM_CAST_CBC                 " },
@@ -809,16 +824,6 @@ void print_mech_info(FILE *f, CK_MECHANISM_TYPE type,
     CKF_GENERATE|CKF_GENERATE_KEY_PAIR|CKF_WRAP|CKF_UNWRAP|
     CKF_DERIVE;
     
-#if 0
-  if (name) {
-    fprintf(f, "%30s ", name);
-  } else {
-    fprintf(f, " Unknown Mechanism (%08lx)  ", type);
-  }
-  fprintf(f, "min:%6lu  max:%6lu  flags:0x%08lX  \n",
-	  minfo->ulMinKeySize, minfo->ulMaxKeySize, minfo->flags);
-  printf("  flags:  %s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
-#else
   if (name) {
     fprintf(f, "%s : ", name);
   } else {
@@ -828,7 +833,6 @@ void print_mech_info(FILE *f, CK_MECHANISM_TYPE type,
 	  (unsigned long) minfo->ulMinKeySize,
 	  (unsigned long) minfo->ulMaxKeySize, minfo->flags);
   fprintf(f, "( %s%s%s%s%s%s%s%s%s%s%s%s%s%s)\n",
-#endif
 	 (minfo->flags & CKF_HW)                ? "Hardware " : "",
 	 (minfo->flags & CKF_ENCRYPT)           ? "Encrypt "  : "",
 	 (minfo->flags & CKF_DECRYPT)           ? "Decrypt "  : "",
