@@ -13,7 +13,7 @@ struct sc_pkcs15_card;
 struct sc_pkcs15_object;
 
 int sc_test_init(int *argc, char *argv[]);
-void sc_test_cleanup();
+void sc_test_cleanup(void);
 void sc_test_print_card(const sc_pkcs15_card_t *);
 void sc_test_print_object(const struct sc_pkcs15_object *);
 
